@@ -1,37 +1,36 @@
 #!/usr/bin/env python
-from saml2.algsupport import algorithm_support_in_metadata
-from saml2.md import AttributeProfile
-from saml2.sigver import security_context
-from saml2.cert import read_cert_from_file
-from saml2.config import Config
-from saml2.validate import valid_instance
-from saml2.time_util import in_a_while
-from saml2.extension import mdui
-from saml2.extension import idpdisc
-from saml2.extension import shibmd
-from saml2.extension import mdattr
-from saml2.extension import sp_type
-from saml2.saml import NAME_FORMAT_URI
-from saml2.saml import AttributeValue
-from saml2.saml import Attribute
-from saml2.attribute_converter import from_local_name
-from saml2 import md, SAMLError
+
 from saml2 import BINDING_HTTP_POST
 from saml2 import BINDING_HTTP_REDIRECT
 from saml2 import BINDING_SOAP
-from saml2 import samlp
+from saml2 import SAMLError
 from saml2 import class_name
-
+from saml2 import md
+from saml2 import samlp
 from saml2 import xmldsig as ds
-import six
-
-from saml2.sigver import pre_signature_part
-
+from saml2.algsupport import algorithm_support_in_metadata
+from saml2.attribute_converter import from_local_name
+from saml2.cert import read_cert_from_file
+from saml2.config import Config
+from saml2.extension import idpdisc
+from saml2.extension import mdattr
+from saml2.extension import mdui
+from saml2.extension import shibmd
+from saml2.extension import sp_type
+from saml2.md import AttributeProfile
 from saml2.s_utils import factory
 from saml2.s_utils import rec_factory
 from saml2.s_utils import sid
+from saml2.saml import NAME_FORMAT_URI
+from saml2.saml import Attribute
+from saml2.saml import AttributeValue
+from saml2.sigver import pre_signature_part
+from saml2.sigver import security_context
+from saml2.time_util import in_a_while
+from saml2.validate import valid_instance
 
-__author__ = 'rolandh'
+
+__author__ = "rolandh"
 
 NSPAIR = {
     "saml2p": "urn:oasis:names:tc:SAML:2.0:protocol",
@@ -54,13 +53,13 @@ DEFAULTS = {
 ORG_ATTR_TRANSL = {
     "organization_name": ("name", md.OrganizationName),
     "organization_display_name": ("display_name", md.OrganizationDisplayName),
-    "organization_url": ("url", md.OrganizationURL)
+    "organization_url": ("url", md.OrganizationURL),
 }
 
 MDNS = '"urn:oasis:names:tc:SAML:2.0:metadata"'
 bMDNS = b'"urn:oasis:names:tc:SAML:2.0:metadata"'
-XMLNSXS = " xmlns:xs=\"http://www.w3.org/2001/XMLSchema\""
-bXMLNSXS = b" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\""
+XMLNSXS = ' xmlns:xs="http://www.w3.org/2001/XMLSchema"'
+bXMLNSXS = b' xmlns:xs="http://www.w3.org/2001/XMLSchema"'
 
 
 def metadata_tostring_fix(desc, nspair, xmlstring=""):
@@ -68,17 +67,27 @@ def metadata_tostring_fix(desc, nspair, xmlstring=""):
         xmlstring = desc.to_string(nspair)
 
     try:
-        if "\"xs:string\"" in xmlstring and XMLNSXS not in xmlstring:
+        if '"xs:string"' in xmlstring and XMLNSXS not in xmlstring:
             xmlstring = xmlstring.replace(MDNS, MDNS + XMLNSXS)
     except TypeError:
-        if b"\"xs:string\"" in xmlstring and bXMLNSXS not in xmlstring:
+        if b'"xs:string"' in xmlstring and bXMLNSXS not in xmlstring:
             xmlstring = xmlstring.replace(bMDNS, bMDNS + bXMLNSXS)
 
     return xmlstring
 
 
-def create_metadata_string(configfile, config=None, valid=None, cert=None,
-                           keyfile=None, mid=None, name=None, sign=None):
+def create_metadata_string(
+    configfile,
+    config=None,
+    valid=None,
+    cert=None,
+    keyfile=None,
+    mid=None,
+    name=None,
+    sign=None,
+    sign_alg=None,
+    digest_alg=None,
+):
     valid_for = 0
     nspair = {"xs": "http://www.w3.org/2001/XMLSchema"}
     # paths = [".", "/opt/local/bin"]
@@ -96,17 +105,17 @@ def create_metadata_string(configfile, config=None, valid=None, cert=None,
     conf = Config()
     conf.key_file = config.key_file or keyfile
     conf.cert_file = config.cert_file or cert
-    conf.debug = 1
     conf.xmlsec_binary = config.xmlsec_binary
     secc = security_context(conf)
 
+    sign_alg = sign_alg or config.signing_algorithm
+    digest_alg = digest_alg or config.digest_algorithm
     if mid:
-        eid, xmldoc = entities_descriptor(eds, valid_for, name, mid,
-                                          sign, secc)
+        eid, xmldoc = entities_descriptor(eds, valid_for, name, mid, sign, secc, sign_alg, digest_alg)
     else:
         eid = eds[0]
         if sign:
-            eid, xmldoc = sign_entity_descriptor(eid, mid, secc)
+            eid, xmldoc = sign_entity_descriptor(eid, mid, secc, sign_alg, digest_alg)
         else:
             xmldoc = None
 
@@ -143,11 +152,10 @@ def do_organization_info(ava):
     for dkey, (ckey, klass) in ORG_ATTR_TRANSL.items():
         if ckey not in ava:
             continue
-        if isinstance(ava[ckey], six.string_types):
+        if isinstance(ava[ckey], str):
             setattr(org, dkey, [_localized_name(ava[ckey], klass)])
         elif isinstance(ava[ckey], list):
-            setattr(org, dkey,
-                    [_localized_name(n, klass) for n in ava[ckey]])
+            setattr(org, dkey, [_localized_name(n, klass) for n in ava[ckey]])
         else:
             setattr(org, dkey, [_localized_name(ava[ckey], klass)])
     return org
@@ -175,12 +183,8 @@ def do_key_descriptor(cert=None, enc_cert=None, use="both"):
         for _cert in cert:
             kd_list.append(
                 md.KeyDescriptor(
-                    key_info=ds.KeyInfo(
-                        x509_data=ds.X509Data(
-                            x509_certificate=ds.X509Certificate(text=_cert)
-                        )
-                    ),
-                    use="signing"
+                    key_info=ds.KeyInfo(x509_data=ds.X509Data(x509_certificate=ds.X509Certificate(text=_cert))),
+                    use="signing",
                 )
             )
     if use in ["both", "encryption"] and enc_cert is not None:
@@ -189,32 +193,23 @@ def do_key_descriptor(cert=None, enc_cert=None, use="both"):
         for _enc_cert in enc_cert:
             kd_list.append(
                 md.KeyDescriptor(
-                    key_info=ds.KeyInfo(
-                        x509_data=ds.X509Data(
-                            x509_certificate=ds.X509Certificate(text=_enc_cert)
-                        )
-                    ),
-                    use="encryption"
+                    key_info=ds.KeyInfo(x509_data=ds.X509Data(x509_certificate=ds.X509Certificate(text=_enc_cert))),
+                    use="encryption",
                 )
             )
     if len(kd_list) == 0 and cert is not None:
         return md.KeyDescriptor(
-            key_info=ds.KeyInfo(
-                x509_data=ds.X509Data(
-                    x509_certificate=ds.X509Certificate(text=cert)
-                )
-            )
+            key_info=ds.KeyInfo(x509_data=ds.X509Data(x509_certificate=ds.X509Certificate(text=cert)))
         )
     return kd_list
 
 
-def do_requested_attribute(attributes, acs, is_required="false",
-                           name_format=NAME_FORMAT_URI):
+def do_requested_attribute(attributes, acs, is_required="false", name_format=NAME_FORMAT_URI):
     lista = []
     for attr in attributes:
         attr = from_local_name(acs, attr, name_format)
         args = {}
-        if isinstance(attr, six.string_types):
+        if isinstance(attr, str):
             args["name"] = attr
         else:
             for key in attr.keyswv():
@@ -227,8 +222,7 @@ def do_requested_attribute(attributes, acs, is_required="false",
 
 def do_uiinfo(_uiinfo):
     uii = mdui.UIInfo()
-    for attr in ['display_name', 'description', "information_url",
-                 'privacy_statement_url']:
+    for attr in ["display_name", "description", "information_url", "privacy_statement_url"]:
         try:
             val = _uiinfo[attr]
         except KeyError:
@@ -236,7 +230,7 @@ def do_uiinfo(_uiinfo):
 
         aclass = uii.child_class(attr)
         inst = getattr(uii, attr)
-        if isinstance(val, six.string_types):
+        if isinstance(val, str):
             ainst = aclass(text=val)
             inst.append(ainst)
         elif isinstance(val, dict):
@@ -246,7 +240,7 @@ def do_uiinfo(_uiinfo):
             inst.append(ainst)
         else:
             for value in val:
-                if isinstance(value, six.string_types):
+                if isinstance(value, str):
                     ainst = aclass(text=value)
                     inst.append(ainst)
                 elif isinstance(value, dict):
@@ -282,11 +276,11 @@ def do_uiinfo(_uiinfo):
         _attr = "keywords"
         val = _uiinfo[_attr]
         inst = getattr(uii, _attr)
-        # list of six.string_types, dictionary or list of dictionaries
+        # list of strings, bytes, dictionary or list of dictionaries
         if isinstance(val, list):
             for value in val:
                 keyw = mdui.Keywords()
-                if isinstance(value, six.string_types):
+                if isinstance(value, str):
                     keyw.text = value
                 elif isinstance(value, dict):
                     keyw.text = " ".join(value["text"])
@@ -314,8 +308,7 @@ def do_uiinfo(_uiinfo):
 
 
 def do_idpdisc(discovery_response):
-    return idpdisc.DiscoveryResponse(index="0", location=discovery_response,
-                                     binding=idpdisc.NAMESPACE)
+    return idpdisc.DiscoveryResponse(index="0", location=discovery_response, binding=idpdisc.NAMESPACE)
 
 
 ENDPOINTS = {
@@ -338,21 +331,13 @@ ENDPOINTS = {
         "single_logout_service": (md.SingleLogoutService, False),
         "manage_name_id_service": (md.ManageNameIDService, False),
         "assertion_id_request_service": (md.AssertionIDRequestService, False),
-        "attribute_service": (md.AttributeService, False)
+        "attribute_service": (md.AttributeService, False),
     },
-    "pdp": {
-        "authz_service": (md.AuthzService, True)
-    },
-    "aq": {
-        "authn_query_service": (md.AuthnQueryService, True)
-    }
+    "pdp": {"authz_service": (md.AuthzService, True)},
+    "aq": {"authn_query_service": (md.AuthnQueryService, True)},
 }
 
-ENDPOINT_EXT = {
-    "sp": {
-        "discovery_response": (idpdisc.DiscoveryResponse, True)
-    }
-}
+ENDPOINT_EXT = {"sp": {"discovery_response": (idpdisc.DiscoveryResponse, True)}}
 
 DEFAULT_BINDING = {
     "assertion_consumer_service": BINDING_HTTP_POST,
@@ -360,14 +345,13 @@ DEFAULT_BINDING = {
     "single_logout_service": BINDING_HTTP_POST,
     "attribute_service": BINDING_SOAP,
     "artifact_resolution_service": BINDING_SOAP,
-    "authn_query_service": BINDING_SOAP
+    "authn_query_service": BINDING_SOAP,
 }
 
 
 def do_extensions(mname, item):
     try:
-        _mod = __import__("saml2.extension.%s" % mname, globals(), locals(),
-                          mname)
+        _mod = __import__(f"saml2.extension.{mname}", globals(), locals(), mname)
     except ImportError:
         return None
     else:
@@ -384,11 +368,11 @@ def _do_nameid_format(cls, conf, typ):
     if not name_id_format:
         return
 
-    if isinstance(name_id_format, six.string_types):
+    if isinstance(name_id_format, str):
         name_id_format = [name_id_format]
 
     formats = [md.NameIDFormat(text=format) for format in name_id_format]
-    setattr(cls, "name_id_format", formats)
+    cls.name_id_format = formats
 
 
 def do_endpoints(conf, endpoints):
@@ -399,20 +383,17 @@ def do_endpoints(conf, endpoints):
             servs = []
             i = 1
             for args in conf[endpoint]:
-                if isinstance(args,
-                              six.string_types):  # Assume it's the location
-                    args = {"location": args,
-                            "binding": DEFAULT_BINDING[endpoint]}
+                if isinstance(args, str):  # Assume it's the location
+                    args = {"location": args, "binding": DEFAULT_BINDING[endpoint]}
                 elif isinstance(args, tuple) or isinstance(args, list):
                     if len(args) == 2:  # (location, binding)
                         args = {"location": args[0], "binding": args[1]}
                     elif len(args) == 3:  # (location, binding, index)
-                        args = {"location": args[0], "binding": args[1],
-                                "index": args[2]}
+                        args = {"location": args[0], "binding": args[1], "index": args[2]}
 
                 if indexed:
                     if "index" not in args:
-                        args["index"] = "%d" % i
+                        args["index"] = f"{int(i)}"
                         i += 1
                     else:
                         try:
@@ -449,14 +430,13 @@ def do_attribute_consuming_service(conf, spsso):
 
     if req:
         requested_attributes.extend(
-            do_requested_attribute(req, acs, is_required="true",
-                                   name_format=req_attr_name_format))
+            do_requested_attribute(req, acs, is_required="true", name_format=req_attr_name_format)
+        )
 
     opt = conf.getattr("optional_attributes", "sp")
 
     if opt:
-        requested_attributes.extend(
-            do_requested_attribute(opt, acs, name_format=req_attr_name_format))
+        requested_attributes.extend(do_requested_attribute(opt, acs, name_format=req_attr_name_format))
 
     try:
         if conf.description:
@@ -486,8 +466,8 @@ def do_attribute_consuming_service(conf, spsso):
             service_name = [md.ServiceName(text="", lang="en")]
 
         ac_serv = md.AttributeConsumingService(
-            index="1", service_name=service_name,
-            requested_attribute=requested_attributes)
+            index="1", service_name=service_name, requested_attribute=requested_attributes
+        )
 
         if service_description:
             ac_serv.service_description = service_description
@@ -512,8 +492,7 @@ def do_spsso_descriptor(conf, cert=None, enc_cert=None):
 
     endps = conf.getattr("endpoints", "sp")
     if endps:
-        for (endpoint, instlist) in do_endpoints(endps,
-                                                 ENDPOINTS["sp"]).items():
+        for (endpoint, instlist) in do_endpoints(endps, ENDPOINTS["sp"]).items():
             setattr(spsso, endpoint, instlist)
 
     ext = do_endpoints(endps, ENDPOINT_EXT["sp"])
@@ -532,8 +511,7 @@ def do_spsso_descriptor(conf, cert=None, enc_cert=None):
 
     if cert or enc_cert:
         metadata_key_usage = conf.metadata_key_usage
-        spsso.key_descriptor = do_key_descriptor(cert=cert, enc_cert=enc_cert,
-                                                 use=metadata_key_usage)
+        spsso.key_descriptor = do_key_descriptor(cert=cert, enc_cert=enc_cert, use=metadata_key_usage)
 
     for key in ["want_assertions_signed", "authn_requests_signed"]:
         try:
@@ -541,7 +519,7 @@ def do_spsso_descriptor(conf, cert=None, enc_cert=None):
             if val is None:
                 setattr(spsso, key, DEFAULT[key])  # default ?!
             else:
-                strval = "{0:>s}".format(str(val))
+                strval = f"{str(val):>s}"
                 setattr(spsso, key, strval.lower())
         except KeyError:
             setattr(spsso, key, DEFAULTS[key])
@@ -555,10 +533,20 @@ def do_idpsso_descriptor(conf, cert=None, enc_cert=None):
     idpsso = md.IDPSSODescriptor()
     idpsso.protocol_support_enumeration = samlp.NAMESPACE
 
+    exts = conf.getattr("extensions", "idp")
+    if exts:
+        if idpsso.extensions is None:
+            idpsso.extensions = md.Extensions()
+
+        for key, val in exts.items():
+            _ext = do_extensions(key, val)
+            if _ext:
+                for _e in _ext:
+                    idpsso.extensions.add_extension_element(_e)
+
     endps = conf.getattr("endpoints", "idp")
     if endps:
-        for (endpoint, instlist) in do_endpoints(endps,
-                                                 ENDPOINTS["idp"]).items():
+        for (endpoint, instlist) in do_endpoints(endps, ENDPOINTS["idp"]).items():
             setattr(idpsso, endpoint, instlist)
 
     _do_nameid_format(idpsso, conf, "idp")
@@ -581,8 +569,7 @@ def do_idpsso_descriptor(conf, cert=None, enc_cert=None):
         idpsso.extensions.add_extension_element(do_uiinfo(ui_info))
 
     if cert or enc_cert:
-        idpsso.key_descriptor = do_key_descriptor(cert, enc_cert,
-                                                  use=conf.metadata_key_usage)
+        idpsso.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
 
     for key in ["want_authn_requests_signed"]:
         # "want_authn_requests_only_with_valid_cert"]:
@@ -591,7 +578,7 @@ def do_idpsso_descriptor(conf, cert=None, enc_cert=None):
             if val is None:
                 setattr(idpsso, key, DEFAULT[key])
             else:
-                setattr(idpsso, key, ("%s" % val).lower())
+                setattr(idpsso, key, (f"{val}").lower())
         except KeyError:
             setattr(idpsso, key, DEFAULTS[key])
 
@@ -602,18 +589,27 @@ def do_aa_descriptor(conf, cert=None, enc_cert=None):
     aad = md.AttributeAuthorityDescriptor()
     aad.protocol_support_enumeration = samlp.NAMESPACE
 
+    exts = conf.getattr("extensions", "aa")
+    if exts:
+        if aad.extensions is None:
+            aad.extensions = md.Extensions()
+
+        for key, val in exts.items():
+            _ext = do_extensions(key, val)
+            if _ext:
+                for _e in _ext:
+                    aad.extensions.add_extension_element(_e)
+
     endps = conf.getattr("endpoints", "aa")
 
     if endps:
-        for (endpoint, instlist) in do_endpoints(endps,
-                                                 ENDPOINTS["aa"]).items():
+        for (endpoint, instlist) in do_endpoints(endps, ENDPOINTS["aa"]).items():
             setattr(aad, endpoint, instlist)
 
     _do_nameid_format(aad, conf, "aa")
 
     if cert or enc_cert:
-        aad.key_descriptor = do_key_descriptor(cert, enc_cert,
-                                               use=conf.metadata_key_usage)
+        aad.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
 
     attributes = conf.getattr("attribute", "aa")
     if attributes:
@@ -632,51 +628,65 @@ def do_aq_descriptor(conf, cert=None, enc_cert=None):
     aqs = md.AuthnAuthorityDescriptor()
     aqs.protocol_support_enumeration = samlp.NAMESPACE
 
+    exts = conf.getattr("extensions", "aa")
+    if exts:
+        if aqs.extensions is None:
+            aqs.extensions = md.Extensions()
+
+        for key, val in exts.items():
+            _ext = do_extensions(key, val)
+            if _ext:
+                for _e in _ext:
+                    aqs.extensions.add_extension_element(_e)
+
     endps = conf.getattr("endpoints", "aq")
 
     if endps:
-        for (endpoint, instlist) in do_endpoints(endps,
-                                                 ENDPOINTS["aq"]).items():
+        for (endpoint, instlist) in do_endpoints(endps, ENDPOINTS["aq"]).items():
             setattr(aqs, endpoint, instlist)
 
     _do_nameid_format(aqs, conf, "aq")
 
     if cert or enc_cert:
-        aqs.key_descriptor = do_key_descriptor(cert, enc_cert,
-                                               use=conf.metadata_key_usage)
+        aqs.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
 
     return aqs
 
 
 def do_pdp_descriptor(conf, cert=None, enc_cert=None):
-    """ Create a Policy Decision Point descriptor """
+    """Create a Policy Decision Point descriptor"""
     pdp = md.PDPDescriptor()
 
     pdp.protocol_support_enumeration = samlp.NAMESPACE
 
+    exts = conf.getattr("extensions", "pdp")
+    if exts:
+        if pdp.extensions is None:
+            pdp.extensions = md.Extensions()
+
+        for key, val in exts.items():
+            _ext = do_extensions(key, val)
+            if _ext:
+                for _e in _ext:
+                    pdp.extensions.add_extension_element(_e)
+
     endps = conf.getattr("endpoints", "pdp")
 
     if endps:
-        for (endpoint, instlist) in do_endpoints(endps,
-                                                 ENDPOINTS["pdp"]).items():
+        for (endpoint, instlist) in do_endpoints(endps, ENDPOINTS["pdp"]).items():
             setattr(pdp, endpoint, instlist)
 
     _do_nameid_format(pdp, conf, "pdp")
 
     if cert:
-        pdp.key_descriptor = do_key_descriptor(cert, enc_cert,
-                                               use=conf.metadata_key_usage)
+        pdp.key_descriptor = do_key_descriptor(cert, enc_cert, use=conf.metadata_key_usage)
 
     return pdp
 
 
 def _add_attr_to_entity_attributes(extensions, attribute):
     try:
-        entity_attributes = next(
-            el
-            for el in extensions.extension_elements
-            if el.tag == mdattr.EntityAttributes.c_tag
-        )
+        entity_attributes = next(el for el in extensions.extension_elements if el.tag == mdattr.EntityAttributes.c_tag)
     except StopIteration:
         entity_attributes = mdattr.EntityAttributes(attribute=attribute)
         extensions.add_extension_element(entity_attributes)
@@ -709,6 +719,17 @@ def entity_descriptor(confd):
     if confd.contact_person is not None:
         entd.contact_person = do_contact_persons_info(confd.contact_person)
 
+    exts = confd.extensions
+    if exts:
+        if not entd.extensions:
+            entd.extensions = md.Extensions()
+
+        for key, val in exts.items():
+            _ext = do_extensions(key, val)
+            if _ext:
+                for _e in _ext:
+                    entd.extensions.add_extension_element(_e)
+
     if confd.entity_attributes:
         if not entd.extensions:
             entd.extensions = md.Extensions()
@@ -717,10 +738,7 @@ def entity_descriptor(confd):
                 name_format=attr.get("format"),
                 name=attr.get("name"),
                 friendly_name=attr.get("friendly_name"),
-                attribute_value=[
-                    AttributeValue(text=value)
-                    for value in attr.get("values", [])
-                ],
+                attribute_value=[AttributeValue(text=value) for value in attr.get("values", [])],
             )
             for attr in confd.entity_attributes
         ]
@@ -741,18 +759,14 @@ def entity_descriptor(confd):
         if not entd.extensions:
             entd.extensions = md.Extensions()
         ava = [AttributeValue(text=c) for c in confd.entity_category]
-        attr = Attribute(
-            attribute_value=ava, name="http://macedir.org/entity-category"
-        )
+        attr = Attribute(attribute_value=ava, name="http://macedir.org/entity-category")
         _add_attr_to_entity_attributes(entd.extensions, attr)
 
     if confd.entity_category_support:
         if not entd.extensions:
             entd.extensions = md.Extensions()
         ava = [AttributeValue(text=c) for c in confd.entity_category_support]
-        attr = Attribute(
-            attribute_value=ava, name="http://macedir.org/entity-category-support"
-        )
+        attr = Attribute(attribute_value=ava, name="http://macedir.org/entity-category-support")
         _add_attr_to_entity_attributes(entd.extensions, attr)
 
     for item in algorithm_support_in_metadata(confd.xmlsec_binary):
@@ -760,8 +774,8 @@ def entity_descriptor(confd):
             entd.extensions = md.Extensions()
         entd.extensions.add_extension_element(item)
 
-    conf_sp_type = confd.getattr('sp_type', 'sp')
-    conf_sp_type_in_md = confd.getattr('sp_type_in_metadata', 'sp')
+    conf_sp_type = confd.getattr("sp_type", "sp")
+    conf_sp_type_in_md = confd.getattr("sp_type_in_metadata", "sp")
     if conf_sp_type and conf_sp_type_in_md is True:
         if not entd.extensions:
             entd.extensions = md.Extensions()
@@ -770,8 +784,7 @@ def entity_descriptor(confd):
 
     serves = confd.serves
     if not serves:
-        raise SAMLError(
-            'No service type ("sp","idp","aa") provided in the configuration')
+        raise SAMLError('No service type ("sp","idp","aa") provided in the configuration')
 
     if "sp" in serves:
         confd.context = "sp"
@@ -781,21 +794,18 @@ def entity_descriptor(confd):
         entd.idpsso_descriptor = do_idpsso_descriptor(confd, mycert, enc_cert)
     if "aa" in serves:
         confd.context = "aa"
-        entd.attribute_authority_descriptor = do_aa_descriptor(confd, mycert,
-                                                               enc_cert)
+        entd.attribute_authority_descriptor = do_aa_descriptor(confd, mycert, enc_cert)
     if "pdp" in serves:
         confd.context = "pdp"
         entd.pdp_descriptor = do_pdp_descriptor(confd, mycert, enc_cert)
     if "aq" in serves:
         confd.context = "aq"
-        entd.authn_authority_descriptor = do_aq_descriptor(confd, mycert,
-                                                           enc_cert)
+        entd.authn_authority_descriptor = do_aq_descriptor(confd, mycert, enc_cert)
 
     return entd
 
 
-def entities_descriptor(eds, valid_for, name, ident, sign, secc, sign_alg=None,
-                        digest_alg=None):
+def entities_descriptor(eds, valid_for, name, ident, sign, secc, sign_alg=None, digest_alg=None):
     entities = md.EntitiesDescriptor(entity_descriptor=eds)
     if valid_for:
         entities.valid_until = in_a_while(hours=valid_for)
@@ -809,18 +819,14 @@ def entities_descriptor(eds, valid_for, name, ident, sign, secc, sign_alg=None,
             ident = sid()
 
         if not secc.key_file:
-            raise SAMLError("If you want to do signing you should define " +
-                            "a key to sign with")
+            raise SAMLError(f"If you want to do signing you should define a key to sign with")
 
         if not secc.my_cert:
-            raise SAMLError("If you want to do signing you should define " +
-                            "where your public key are")
+            raise SAMLError(f"If you want to do signing you should define where your public key are")
 
-        entities.signature = pre_signature_part(ident, secc.my_cert, 1,
-                                                sign_alg=sign_alg,
-                                                digest_alg=digest_alg)
+        entities.signature = pre_signature_part(ident, secc.my_cert, 1, sign_alg=sign_alg, digest_alg=digest_alg)
         entities.id = ident
-        xmldoc = secc.sign_statement("%s" % entities, class_name(entities))
+        xmldoc = secc.sign_statement(f"{entities}", class_name(entities))
         entities = md.entities_descriptor_from_string(xmldoc)
     else:
         xmldoc = None
@@ -842,6 +848,6 @@ def sign_entity_descriptor(edesc, ident, secc, sign_alg=None, digest_alg=None):
 
     edesc.signature = pre_signature_part(ident, secc.my_cert, 1, sign_alg=sign_alg, digest_alg=digest_alg)
     edesc.id = ident
-    xmldoc = secc.sign_statement("%s" % edesc, class_name(edesc))
+    xmldoc = secc.sign_statement(f"{edesc}", class_name(edesc))
     edesc = md.entity_descriptor_from_string(xmldoc)
     return edesc, xmldoc
